import __builtin__

from pds4_logging import logger_init

# Safe import of numpy (not required)
try:
    import numpy as np
except ImportError:
    pass

# Initialize the logger
logger = logger_init()

#################################


# Converts byte_data to data_type corresponding to the appropriate types (built-in types if use_numpy is False, or
# numpy types if use_numpy is True). Note that this method has been optimized for speed, take care when modifying it
def data_type_convert(data_type, byte_data, use_numpy=False):

    python_type = {'ASCII_Boolean': ('bool',),
                   'UTF8_String': ('unicode', 'utf-8'),
                   'ASCII_Numeric_Base2': ('int', 2),
                   'ASCII_Numeric_Base8': ('int', 8),
                   'ASCII_Numeric_Base16': ('int', 16),
                   }.get(data_type, None)

    if not use_numpy:

        python_type = {'ASCII_Real': ('float',),
                       'ASCII_Integer': ('int',),
                       'ASCII_NonNegative_Integer': ('long',),
                       }.get(data_type, ('str',))

    if python_type is not None:

        cast_func = getattr(__builtin__, python_type[0])

        if python_type[0] == 'unicode':

            # Special handling for unicode
            for i, datum in enumerate(byte_data):
                byte_data[i] = cast_func(datum).encode('utf-8')

        elif python_type[0] == 'bool':

            # Special handling for boolean due to e.g. bool('false') = True
            for i, datum in enumerate(byte_data):
                if datum in ['true', '1']:
                    byte_data[i] = True
                else:
                    byte_data[i] = False

        elif python_type[0]:

            # Normal conversion for most data types
            args = python_type[1:]

            for i, datum in enumerate(byte_data):
                byte_data[i] = cast_func(datum, *args)

    # Convert to numpy data types if requested
    if use_numpy:

        if data_type == 'ASCII_Real':
            byte_data = np.asarray(byte_data, dtype=np.float64)
            byte_data = np.asarray(byte_data, dtype=np.float64)
        elif data_type == 'ASCII_Integer':
            byte_data = np.asarray(byte_data, dtype=np.int64)
        elif data_type == 'ASCII_NonNegative_Integer':
            byte_data = np.asarray(byte_data, dtype=np.uint64)
        elif data_type == 'ASCII_Numeric_Base2':
            byte_data = np.asarray(byte_data, dtype=np.int64)
        elif data_type == 'ASCII_Numeric_Base8':
            byte_data = np.asarray(byte_data, dtype=np.int64)
        elif data_type == 'ASCII_Numeric_Base16':
            byte_data = np.asarray(byte_data, dtype=np.int64)
        elif data_type == 'ASCII_Boolean':
            byte_data = np.asarray(byte_data, dtype=np.bool)
        else:
            byte_data = np.asarray(byte_data, dtype=np.str)

    return byte_data